unit MarkdownHTMLEntities;

// from https://html.spec.whatwg.org/entities.json

{
Copyright (c) 2011+, Health Intersections Pty Ltd (http://www.healthintersections.com.au)
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 * Neither the name of HL7 nor the names of its contributors may be used to
   endorse or promote products derived from this software without specific
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 'AS IS' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
}
{$IFDEF FPC}
{$WARN 4108 off : Unicode constant cast with potential data loss}
{$ENDIF}

interface

{$IFDEF FPC}{$MODE DELPHI}{$ENDIF}

uses
  SysUtils, Generics.Collections;

procedure registerEntities(dict : TDictionary<String,String>);

implementation

procedure registerEntities(dict : TDictionary<String,String>);
  procedure add(name, value : {$IFDEF FPC}UTF8String{$ELSE}String{$ENDIF});
  begin
    dict.add(name, value);
  end;
begin
  add('&AElig;', #$00C6);
  add('&AMP;', #$0026);
  add('&Aacute;', #$00C1);
  add('&Abreve;', #$0102);
  add('&Acirc;', #$00C2);
  add('&Acy;', #$0410);
  add('&Afr;', #$D835#$DD04);
  add('&Agrave;', #$00C0);
  add('&Alpha;', #$0391);
  add('&Amacr;', #$0100);
  add('&And;', #$2A53);
  add('&Aogon;', #$0104);
  add('&Aopf;', #$D835#$DD38);
  add('&ApplyFunction;', #$2061);
  add('&Aring;', #$00C5);
  add('&Ascr;', #$D835#$DC9C);
  add('&Assign;', #$2254);
  add('&Atilde;', #$00C3);
  add('&Auml;', #$00C4);
  add('&Backslash;', #$2216);
  add('&Barv;', #$2AE7);
  add('&Barwed;', #$2306);
  add('&Bcy;', #$0411);
  add('&Because;', #$2235);
  add('&Bernoullis;', #$212C);
  add('&Beta;', #$0392);
  add('&Bfr;', #$D835#$DD05);
  add('&Bopf;', #$D835#$DD39);
  add('&Breve;', #$02D8);
  add('&Bscr;', #$212C);
  add('&Bumpeq;', #$224E);
  add('&CHcy;', #$0427);
  add('&COPY;', #$00A9);
  add('&Cacute;', #$0106);
  add('&Cap;', #$22D2);
  add('&CapitalDifferentialD;', #$2145);
  add('&Cayleys;', #$212D);
  add('&Ccaron;', #$010C);
  add('&Ccedil;', #$00C7);
  add('&Ccirc;', #$0108);
  add('&Cconint;', #$2230);
  add('&Cdot;', #$010A);
  add('&Cedilla;', #$00B8);
  add('&CenterDot;', #$00B7);
  add('&Cfr;', #$212D);
  add('&Chi;', #$03A7);
  add('&CircleDot;', #$2299);
  add('&CircleMinus;', #$2296);
  add('&CirclePlus;', #$2295);
  add('&CircleTimes;', #$2297);
  add('&ClockwiseContourIntegral;', #$2232);
  add('&CloseCurlyDoubleQuote;', #$201D);
  add('&CloseCurlyQuote;', #$2019);
  add('&Colon;', #$2237);
  add('&Colone;', #$2A74);
  add('&Congruent;', #$2261);
  add('&Conint;', #$222F);
  add('&ContourIntegral;', #$222E);
  add('&Copf;', #$2102);
  add('&Coproduct;', #$2210);
  add('&CounterClockwiseContourIntegral;', #$2233);
  add('&Cross;', #$2A2F);
  add('&Cscr;', #$D835#$DC9E);
  add('&Cup;', #$22D3);
  add('&CupCap;', #$224D);
  add('&DD;', #$2145);
  add('&DDotrahd;', #$2911);
  add('&DJcy;', #$0402);
  add('&DScy;', #$0405);
  add('&DZcy;', #$040F);
  add('&Dagger;', #$2021);
  add('&Darr;', #$21A1);
  add('&Dashv;', #$2AE4);
  add('&Dcaron;', #$010E);
  add('&Dcy;', #$0414);
  add('&Del;', #$2207);
  add('&Delta;', #$0394);
  add('&Dfr;', #$D835#$DD07);
  add('&DiacriticalAcute;', #$00B4);
  add('&DiacriticalDot;', #$02D9);
  add('&DiacriticalDoubleAcute;', #$02DD);
  add('&DiacriticalGrave;', #$0060);
  add('&DiacriticalTilde;', #$02DC);
  add('&Diamond;', #$22C4);
  add('&DifferentialD;', #$2146);
  add('&Dopf;', #$D835#$DD3B);
  add('&Dot;', #$00A8);
  add('&DotDot;', #$20DC);
  add('&DotEqual;', #$2250);
  add('&DoubleContourIntegral;', #$222F);
  add('&DoubleDot;', #$00A8);
  add('&DoubleDownArrow;', #$21D3);
  add('&DoubleLeftArrow;', #$21D0);
  add('&DoubleLeftRightArrow;', #$21D4);
  add('&DoubleLeftTee;', #$2AE4);
  add('&DoubleLongLeftArrow;', #$27F8);
  add('&DoubleLongLeftRightArrow;', #$27FA);
  add('&DoubleLongRightArrow;', #$27F9);
  add('&DoubleRightArrow;', #$21D2);
  add('&DoubleRightTee;', #$22A8);
  add('&DoubleUpArrow;', #$21D1);
  add('&DoubleUpDownArrow;', #$21D5);
  add('&DoubleVerticalBar;', #$2225);
  add('&DownArrow;', #$2193);
  add('&DownArrowBar;', #$2913);
  add('&DownArrowUpArrow;', #$21F5);
  add('&DownBreve;', #$0311);
  add('&DownLeftRightVector;', #$2950);
  add('&DownLeftTeeVector;', #$295E);
  add('&DownLeftVector;', #$21BD);
  add('&DownLeftVectorBar;', #$2956);
  add('&DownRightTeeVector;', #$295F);
  add('&DownRightVector;', #$21C1);
  add('&DownRightVectorBar;', #$2957);
  add('&DownTee;', #$22A4);
  add('&DownTeeArrow;', #$21A7);
  add('&Downarrow;', #$21D3);
  add('&Dscr;', #$D835#$DC9F);
  add('&Dstrok;', #$0110);
  add('&ENG;', #$014A);
  add('&ETH;', #$00D0);
  add('&Eacute;', #$00C9);
  add('&Ecaron;', #$011A);
  add('&Ecirc;', #$00CA);
  add('&Ecy;', #$042D);
  add('&Edot;', #$0116);
  add('&Efr;', #$D835#$DD08);
  add('&Egrave;', #$00C8);
  add('&Element;', #$2208);
  add('&Emacr;', #$0112);
  add('&EmptySmallSquare;', #$25FB);
  add('&EmptyVerySmallSquare;', #$25AB);
  add('&Eogon;', #$0118);
  add('&Eopf;', #$D835#$DD3C);
  add('&Epsilon;', #$0395);
  add('&Equal;', #$2A75);
  add('&EqualTilde;', #$2242);
  add('&Equilibrium;', #$21CC);
  add('&Escr;', #$2130);
  add('&Esim;', #$2A73);
  add('&Eta;', #$0397);
  add('&Euml;', #$00CB);
  add('&Exists;', #$2203);
  add('&ExponentialE;', #$2147);
  add('&Fcy;', #$0424);
  add('&Ffr;', #$D835#$DD09);
  add('&FilledSmallSquare;', #$25FC);
  add('&FilledVerySmallSquare;', #$25AA);
  add('&Fopf;', #$D835#$DD3D);
  add('&ForAll;', #$2200);
  add('&Fouriertrf;', #$2131);
  add('&Fscr;', #$2131);
  add('&GJcy;', #$0403);
  add('&GT;', #$003E);
  add('&Gamma;', #$0393);
  add('&Gammad;', #$03DC);
  add('&Gbreve;', #$011E);
  add('&Gcedil;', #$0122);
  add('&Gcirc;', #$011C);
  add('&Gcy;', #$0413);
  add('&Gdot;', #$0120);
  add('&Gfr;', #$D835#$DD0A);
  add('&Gg;', #$22D9);
  add('&Gopf;', #$D835#$DD3E);
  add('&GreaterEqual;', #$2265);
  add('&GreaterEqualLess;', #$22DB);
  add('&GreaterFullEqual;', #$2267);
  add('&GreaterGreater;', #$2AA2);
  add('&GreaterLess;', #$2277);
  add('&GreaterSlantEqual;', #$2A7E);
  add('&GreaterTilde;', #$2273);
  add('&Gscr;', #$D835#$DCA2);
  add('&Gt;', #$226B);
  add('&HARDcy;', #$042A);
  add('&Hacek;', #$02C7);
  add('&Hat;', #$005E);
  add('&Hcirc;', #$0124);
  add('&Hfr;', #$210C);
  add('&HilbertSpace;', #$210B);
  add('&Hopf;', #$210D);
  add('&HorizontalLine;', #$2500);
  add('&Hscr;', #$210B);
  add('&Hstrok;', #$0126);
  add('&HumpDownHump;', #$224E);
  add('&HumpEqual;', #$224F);
  add('&IEcy;', #$0415);
  add('&IJlig;', #$0132);
  add('&IOcy;', #$0401);
  add('&Iacute;', #$00CD);
  add('&Icirc;', #$00CE);
  add('&Icy;', #$0418);
  add('&Idot;', #$0130);
  add('&Ifr;', #$2111);
  add('&Igrave;', #$00CC);
  add('&Im;', #$2111);
  add('&Imacr;', #$012A);
  add('&ImaginaryI;', #$2148);
  add('&Implies;', #$21D2);
  add('&Int;', #$222C);
  add('&Integral;', #$222B);
  add('&Intersection;', #$22C2);
  add('&InvisibleComma;', #$2063);
  add('&InvisibleTimes;', #$2062);
  add('&Iogon;', #$012E);
  add('&Iopf;', #$D835#$DD40);
  add('&Iota;', #$0399);
  add('&Iscr;', #$2110);
  add('&Itilde;', #$0128);
  add('&Iukcy;', #$0406);
  add('&Iuml;', #$00CF);
  add('&Jcirc;', #$0134);
  add('&Jcy;', #$0419);
  add('&Jfr;', #$D835#$DD0D);
  add('&Jopf;', #$D835#$DD41);
  add('&Jscr;', #$D835#$DCA5);
  add('&Jsercy;', #$0408);
  add('&Jukcy;', #$0404);
  add('&KHcy;', #$0425);
  add('&KJcy;', #$040C);
  add('&Kappa;', #$039A);
  add('&Kcedil;', #$0136);
  add('&Kcy;', #$041A);
  add('&Kfr;', #$D835#$DD0E);
  add('&Kopf;', #$D835#$DD42);
  add('&Kscr;', #$D835#$DCA6);
  add('&LJcy;', #$0409);
  add('&LT;', #$003C);
  add('&Lacute;', #$0139);
  add('&Lambda;', #$039B);
  add('&Lang;', #$27EA);
  add('&Laplacetrf;', #$2112);
  add('&Larr;', #$219E);
  add('&Lcaron;', #$013D);
  add('&Lcedil;', #$013B);
  add('&Lcy;', #$041B);
  add('&LeftAngleBracket;', #$27E8);
  add('&LeftArrow;', #$2190);
  add('&LeftArrowBar;', #$21E4);
  add('&LeftArrowRightArrow;', #$21C6);
  add('&LeftCeiling;', #$2308);
  add('&LeftDoubleBracket;', #$27E6);
  add('&LeftDownTeeVector;', #$2961);
  add('&LeftDownVector;', #$21C3);
  add('&LeftDownVectorBar;', #$2959);
  add('&LeftFloor;', #$230A);
  add('&LeftRightArrow;', #$2194);
  add('&LeftRightVector;', #$294E);
  add('&LeftTee;', #$22A3);
  add('&LeftTeeArrow;', #$21A4);
  add('&LeftTeeVector;', #$295A);
  add('&LeftTriangle;', #$22B2);
  add('&LeftTriangleBar;', #$29CF);
  add('&LeftTriangleEqual;', #$22B4);
  add('&LeftUpDownVector;', #$2951);
  add('&LeftUpTeeVector;', #$2960);
  add('&LeftUpVector;', #$21BF);
  add('&LeftUpVectorBar;', #$2958);
  add('&LeftVector;', #$21BC);
  add('&LeftVectorBar;', #$2952);
  add('&Leftarrow;', #$21D0);
  add('&Leftrightarrow;', #$21D4);
  add('&LessEqualGreater;', #$22DA);
  add('&LessFullEqual;', #$2266);
  add('&LessGreater;', #$2276);
  add('&LessLess;', #$2AA1);
  add('&LessSlantEqual;', #$2A7D);
  add('&LessTilde;', #$2272);
  add('&Lfr;', #$D835#$DD0F);
  add('&Ll;', #$22D8);
  add('&Lleftarrow;', #$21DA);
  add('&Lmidot;', #$013F);
  add('&LongLeftArrow;', #$27F5);
  add('&LongLeftRightArrow;', #$27F7);
  add('&LongRightArrow;', #$27F6);
  add('&Longleftarrow;', #$27F8);
  add('&Longleftrightarrow;', #$27FA);
  add('&Longrightarrow;', #$27F9);
  add('&Lopf;', #$D835#$DD43);
  add('&LowerLeftArrow;', #$2199);
  add('&LowerRightArrow;', #$2198);
  add('&Lscr;', #$2112);
  add('&Lsh;', #$21B0);
  add('&Lstrok;', #$0141);
  add('&Lt;', #$226A);
  add('&Map;', #$2905);
  add('&Mcy;', #$041C);
  add('&MediumSpace;', #$205F);
  add('&Mellintrf;', #$2133);
  add('&Mfr;', #$D835#$DD10);
  add('&MinusPlus;', #$2213);
  add('&Mopf;', #$D835#$DD44);
  add('&Mscr;', #$2133);
  add('&Mu;', #$039C);
  add('&NJcy;', #$040A);
  add('&Nacute;', #$0143);
  add('&Ncaron;', #$0147);
  add('&Ncedil;', #$0145);
  add('&Ncy;', #$041D);
  add('&NegativeMediumSpace;', #$200B);
  add('&NegativeThickSpace;', #$200B);
  add('&NegativeThinSpace;', #$200B);
  add('&NegativeVeryThinSpace;', #$200B);
  add('&NestedGreaterGreater;', #$226B);
  add('&NestedLessLess;', #$226A);
  add('&NewLine;', #$000A);
  add('&Nfr;', #$D835#$DD11);
  add('&NoBreak;', #$2060);
  add('&NonBreakingSpace;', #$00A0);
  add('&Nopf;', #$2115);
  add('&Not;', #$2AEC);
  add('&NotCongruent;', #$2262);
  add('&NotCupCap;', #$226D);
  add('&NotDoubleVerticalBar;', #$2226);
  add('&NotElement;', #$2209);
  add('&NotEqual;', #$2260);
  add('&NotEqualTilde;', #$2242#$0338);
  add('&NotExists;', #$2204);
  add('&NotGreater;', #$226F);
  add('&NotGreaterEqual;', #$2271);
  add('&NotGreaterFullEqual;', #$2267#$033);
  add('&NotGreaterGreater;', #$226B#$033);
  add('&NotGreaterLess;', #$2279);
  add('&NotGreaterSlantEqual;', #$2A7E#$0338);
  add('&NotGreaterTilde;', #$2275);
  add('&NotHumpDownHump;', #$224E#$0338);
  add('&NotHumpEqual;', #$224F#$0338);
  add('&NotLeftTriangle;', #$22EA);
  add('&NotLeftTriangleBar;', #$29CF#$0338);
  add('&NotLeftTriangleEqual;', #$22EC);
  add('&NotLess;', #$226E);
  add('&NotLessEqual;', #$2270);
  add('&NotLessGreater;', #$2278);
  add('&NotLessLess;', #$226A#$0338);
  add('&NotLessSlantEqual;', #$2A7D#$0338);
  add('&NotLessTilde;', #$2274);
  add('&NotNestedGreaterGreater;', #$2AA2#$0338);
  add('&NotNestedLessLess;', #$2AA1#$0338);
  add('&NotPrecedes;', #$2280);
  add('&NotPrecedesEqual;', #$2AAF#$0338);
  add('&NotPrecedesSlantEqual;', #$22E0);
  add('&NotReverseElement;', #$220C);
  add('&NotRightTriangle;', #$22EB);
  add('&NotRightTriangleBar;', #$29D0#$0338);
  add('&NotRightTriangleEqual;', #$22ED);
  add('&NotSquareSubset;', #$228F#$0338);
  add('&NotSquareSubsetEqual;', #$22E2);
  add('&NotSquareSuperset;', #$2290#$0338);
  add('&NotSquareSupersetEqual;', #$22E3);
  add('&NotSubset;', #$2282#$20D2);
  add('&NotSubsetEqual;', #$2288);
  add('&NotSucceeds;', #$2281);
  add('&NotSucceedsEqual;', #$2AB0#$0338);
  add('&NotSucceedsSlantEqual;', #$22E1);
  add('&NotSucceedsTilde;', #$227F#$0338);
  add('&NotSuperset;', #$2283#$20D2);
  add('&NotSupersetEqual;', #$2289);
  add('&NotTilde;', #$2241);
  add('&NotTildeEqual;', #$2244);
  add('&NotTildeFullEqual;', #$2247);
  add('&NotTildeTilde;', #$2249);
  add('&NotVerticalBar;', #$2224);
  add('&Nscr;', #$D835#$DCA9);
  add('&Ntilde;', #$00D1);
  add('&Nu;', #$039D);
  add('&OElig;', #$0152);
  add('&Oacute;', #$00D3);
  add('&Ocirc;', #$00D4);
  add('&Ocy;', #$041E);
  add('&Odblac;', #$0150);
  add('&Ofr;', #$D835#$DD12);
  add('&Ograve;', #$00D2);
  add('&Omacr;', #$014C);
  add('&Omega;', #$03A9);
  add('&Omicron;', #$039F);
  add('&Oopf;', #$D835#$DD46);
  add('&OpenCurlyDoubleQuote;', #$201C);
  add('&OpenCurlyQuote;', #$2018);
  add('&Or;', #$2A54);
  add('&Oscr;', #$D835#$DCAA);
  add('&Oslash;', #$00D8);
  add('&Otilde;', #$00D5);
  add('&Otimes;', #$2A37);
  add('&Ouml;', #$00D6);
  add('&OverBar;', #$203E);
  add('&OverBrace;', #$23DE);
  add('&OverBracket;', #$23B4);
  add('&OverParenthesis;', #$23DC);
  add('&PartialD;', #$2202);
  add('&Pcy;', #$041F);
  add('&Pfr;', #$D835#$DD13);
  add('&Phi;', #$03A6);
  add('&Pi;', #$03A0);
  add('&PlusMinus;', #$00B1);
  add('&Poincareplane;', #$210C);
  add('&Popf;', #$2119);
  add('&Pr;', #$2ABB);
  add('&Precedes;', #$227A);
  add('&PrecedesEqual;', #$2AAF);
  add('&PrecedesSlantEqual;', #$227C);
  add('&PrecedesTilde;', #$227E);
  add('&Prime;', #$2033);
  add('&Product;', #$220F);
  add('&Proportion;', #$2237);
  add('&Proportional;', #$221D);
  add('&Pscr;', #$D835#$DCAB);
  add('&Psi;', #$03A8);
  add('&QUOT;', #$0022);
  add('&Qfr;', #$D835#$DD14);
  add('&Qopf;', #$211A);
  add('&Qscr;', #$D835#$DCAC);
  add('&RBarr;', #$2910);
  add('&REG;', #$00AE);
  add('&Racute;', #$0154);
  add('&Rang;', #$27EB);
  add('&Rarr;', #$21A0);
  add('&Rarrtl;', #$2916);
  add('&Rcaron;', #$0158);
  add('&Rcedil;', #$0156);
  add('&Rcy;', #$0420);
  add('&Re;', #$211C);
  add('&ReverseElement;', #$220B);
  add('&ReverseEquilibrium;', #$21CB);
  add('&ReverseUpEquilibrium;', #$296F);
  add('&Rfr;', #$211C);
  add('&Rho;', #$03A1);
  add('&RightAngleBracket;', #$27E9);
  add('&RightArrow;', #$2192);
  add('&RightArrowBar;', #$21E5);
  add('&RightArrowLeftArrow;', #$21C4);
  add('&RightCeiling;', #$2309);
  add('&RightDoubleBracket;', #$27E7);
  add('&RightDownTeeVector;', #$295D);
  add('&RightDownVector;', #$21C2);
  add('&RightDownVectorBar;', #$2955);
  add('&RightFloor;', #$230B);
  add('&RightTee;', #$22A2);
  add('&RightTeeArrow;', #$21A6);
  add('&RightTeeVector;', #$295B);
  add('&RightTriangle;', #$22B3);
  add('&RightTriangleBar;', #$29D0);
  add('&RightTriangleEqual;', #$22B5);
  add('&RightUpDownVector;', #$294F);
  add('&RightUpTeeVector;', #$295C);
  add('&RightUpVector;', #$21BE);
  add('&RightUpVectorBar;', #$2954);
  add('&RightVector;', #$21C0);
  add('&RightVectorBar;', #$2953);
  add('&Rightarrow;', #$21D2);
  add('&Ropf;', #$211D);
  add('&RoundImplies;', #$2970);
  add('&Rrightarrow;', #$21DB);
  add('&Rscr;', #$211B);
  add('&Rsh;', #$21B1);
  add('&RuleDelayed;', #$29F4);
  add('&SHCHcy;', #$0429);
  add('&SHcy;', #$0428);
  add('&SOFTcy;', #$042C);
  add('&Sacute;', #$015A);
  add('&Sc;', #$2ABC);
  add('&Scaron;', #$0160);
  add('&Scedil;', #$015E);
  add('&Scirc;', #$015C);
  add('&Scy;', #$0421);
  add('&Sfr;', #$D835#$DD16);
  add('&ShortDownArrow;', #$2193);
  add('&ShortLeftArrow;', #$2190);
  add('&ShortRightArrow;', #$2192);
  add('&ShortUpArrow;', #$2191);
  add('&Sigma;', #$03A3);
  add('&SmallCircle;', #$2218);
  add('&Sopf;', #$D835#$DD4A);
  add('&Sqrt;', #$221A);
  add('&Square;', #$25A1);
  add('&SquareIntersection;', #$2293);
  add('&SquareSubset;', #$228F);
  add('&SquareSubsetEqual;', #$2291);
  add('&SquareSuperset;', #$2290);
  add('&SquareSupersetEqual;', #$2292);
  add('&SquareUnion;', #$2294);
  add('&Sscr;', #$D835#$DCAE);
  add('&Star;', #$22C6);
  add('&Sub;', #$22D0);
  add('&Subset;', #$22D0);
  add('&SubsetEqual;', #$2286);
  add('&Succeeds;', #$227B);
  add('&SucceedsEqual;', #$2AB0);
  add('&SucceedsSlantEqual;', #$227D);
  add('&SucceedsTilde;', #$227F);
  add('&SuchThat;', #$220B);
  add('&Sum;', #$2211);
  add('&Sup;', #$22D1);
  add('&Superset;', #$2283);
  add('&SupersetEqual;', #$2287);
  add('&Supset;', #$22D1);
  add('&THORN;', #$00DE);
  add('&TRADE;', #$2122);
  add('&TSHcy;', #$040B);
  add('&TScy;', #$0426);
  add('&Tab;', #$0009);
  add('&Tau;', #$03A4);
  add('&Tcaron;', #$0164);
  add('&Tcedil;', #$0162);
  add('&Tcy;', #$0422);
  add('&Tfr;', #$D835#$DD17);
  add('&Therefore;', #$2234);
  add('&Theta;', #$0398);
  add('&ThickSpace;', #$205F#$200A);
  add('&ThinSpace;', #$2009);
  add('&Tilde;', #$223C);
  add('&TildeEqual;', #$2243);
  add('&TildeFullEqual;', #$2245);
  add('&TildeTilde;', #$2248);
  add('&Topf;', #$D835#$DD4B);
  add('&TripleDot;', #$20DB);
  add('&Tscr;', #$D835#$DCAF);
  add('&Tstrok;', #$0166);
  add('&Uacute;', #$00DA);
  add('&Uarr;', #$219F);
  add('&Uarrocir;', #$2949);
  add('&Ubrcy;', #$040E);
  add('&Ubreve;', #$016C);
  add('&Ucirc;', #$00DB);
  add('&Ucy;', #$0423);
  add('&Udblac;', #$0170);
  add('&Ufr;', #$D835#$DD18);
  add('&Ugrave;', #$00D9);
  add('&Umacr;', #$016A);
  add('&UnderBar;', #$005F);
  add('&UnderBrace;', #$23DF);
  add('&UnderBracket;', #$23B5);
  add('&UnderParenthesis;', #$23DD);
  add('&Union;', #$22C3);
  add('&UnionPlus;', #$228E);
  add('&Uogon;', #$0172);
  add('&Uopf;', #$D835#$DD4C);
  add('&UpArrow;', #$2191);
  add('&UpArrowBar;', #$2912);
  add('&UpArrowDownArrow;', #$21C5);
  add('&UpDownArrow;', #$2195);
  add('&UpEquilibrium;', #$296E);
  add('&UpTee;', #$22A5);
  add('&UpTeeArrow;', #$21A5);
  add('&Uparrow;', #$21D1);
  add('&Updownarrow;', #$21D5);
  add('&UpperLeftArrow;', #$2196);
  add('&UpperRightArrow;', #$2197);
  add('&Upsi;', #$03D2);
  add('&Upsilon;', #$03A5);
  add('&Uring;', #$016E);
  add('&Uscr;', #$D835#$DCB0);
  add('&Utilde;', #$0168);
  add('&Uuml;', #$00DC);
  add('&VDash;', #$22AB);
  add('&Vbar;', #$2AEB);
  add('&Vcy;', #$0412);
  add('&Vdash;', #$22A9);
  add('&Vdashl;', #$2AE6);
  add('&Vee;', #$22C1);
  add('&Verbar;', #$2016);
  add('&Vert;', #$2016);
  add('&VerticalBar;', #$2223);
  add('&VerticalLine;', #$007C);
  add('&VerticalSeparator;', #$2758);
  add('&VerticalTilde;', #$2240);
  add('&VeryThinSpace;', #$200A);
  add('&Vfr;', #$D835#$DD19);
  add('&Vopf;', #$D835#$DD4D);
  add('&Vscr;', #$D835#$DCB1);
  add('&Vvdash;', #$22AA);
  add('&Wcirc;', #$0174);
  add('&Wedge;', #$22C0);
  add('&Wfr;', #$D835#$DD1A);
  add('&Wopf;', #$D835#$DD4E);
  add('&Wscr;', #$D835#$DCB2);
  add('&Xfr;', #$D835#$DD1B);
  add('&Xi;', #$039E);
  add('&Xopf;', #$D835#$DD4F);
  add('&Xscr;', #$D835#$DCB3);
  add('&YAcy;', #$042F);
  add('&YIcy;', #$0407);
  add('&YUcy;', #$042E);
  add('&Yacute;', #$00DD);
  add('&Ycirc;', #$0176);
  add('&Ycy;', #$042B);
  add('&Yfr;', #$D835#$DD1C);
  add('&Yopf;', #$D835#$DD50);
  add('&Yscr;', #$D835#$DCB4);
  add('&Yuml;', #$0178);
  add('&ZHcy;', #$0416);
  add('&Zacute;', #$0179);
  add('&Zcaron;', #$017D);
  add('&Zcy;', #$0417);
  add('&Zdot;', #$017B);
  add('&ZeroWidthSpace;', #$200B);
  add('&Zeta;', #$0396);
  add('&Zfr;', #$2128);
  add('&Zopf;', #$2124);
  add('&Zscr;', #$D835#$DCB5);
  add('&aacute;', #$00E1);
  add('&abreve;', #$0103);
  add('&ac;', #$223E);
  add('&acE;', #$223E#$0333);
  add('&acd;', #$223F);
  add('&acirc;', #$00E2);
  add('&acute;', #$00B4);
  add('&acy;', #$0430);
  add('&aelig;', #$00E6);
  add('&af;', #$2061);
  add('&afr;', #$D835#$DD1E);
  add('&agrave;', #$00E0);
  add('&alefsym;', #$2135);
  add('&aleph;', #$2135);
  add('&alpha;', #$03B1);
  add('&amacr;', #$0101);
  add('&amalg;', #$2A3F);
  add('&amp;', #$0026);
  add('&and;', #$2227);
  add('&andand;', #$2A55);
  add('&andd;', #$2A5C);
  add('&andslope;', #$2A58);
  add('&andv;', #$2A5A);
  add('&ang;', #$2220);
  add('&ange;', #$29A4);
  add('&angle;', #$2220);
  add('&angmsd;', #$2221);
  add('&angmsdaa;', #$29A8);
  add('&angmsdab;', #$29A9);
  add('&angmsdac;', #$29AA);
  add('&angmsdad;', #$29AB);
  add('&angmsdae;', #$29AC);
  add('&angmsdaf;', #$29AD);
  add('&angmsdag;', #$29AE);
  add('&angmsdah;', #$29AF);
  add('&angrt;', #$221F);
  add('&angrtvb;', #$22BE);
  add('&angrtvbd;', #$299D);
  add('&angsph;', #$2222);
  add('&angst;', #$00C5);
  add('&angzarr;', #$237C);
  add('&aogon;', #$0105);
  add('&aopf;', #$D835#$DD52);
  add('&ap;', #$2248);
  add('&apE;', #$2A70);
  add('&apacir;', #$2A6F);
  add('&ape;', #$224A);
  add('&apid;', #$224B);
  add('&apos;', #$0027);
  add('&approx;', #$2248);
  add('&approxeq;', #$224A);
  add('&aring;', #$00E5);
  add('&ascr;', #$D835#$DCB6);
  add('&ast;', #$002A);
  add('&asymp;', #$2248);
  add('&asympeq;', #$224D);
  add('&atilde;', #$00E3);
  add('&auml;', #$00E4);
  add('&awconint;', #$2233);
  add('&awint;', #$2A11);
  add('&bNot;', #$2AED);
  add('&backcong;', #$224C);
  add('&backepsilon;', #$03F6);
  add('&backprime;', #$2035);
  add('&backsim;', #$223D);
  add('&backsimeq;', #$22CD);
  add('&barvee;', #$22BD);
  add('&barwed;', #$2305);
  add('&barwedge;', #$2305);
  add('&bbrk;', #$23B5);
  add('&bbrktbrk;', #$23B6);
  add('&bcong;', #$224C);
  add('&bcy;', #$0431);
  add('&bdquo;', #$201E);
  add('&becaus;', #$2235);
  add('&because;', #$2235);
  add('&bemptyv;', #$29B0);
  add('&bepsi;', #$03F6);
  add('&bernou;', #$212C);
  add('&beta;', #$03B2);
  add('&beth;', #$2136);
  add('&between;', #$226C);
  add('&bfr;', #$D835#$DD1F);
  add('&bigcap;', #$22C2);
  add('&bigcirc;', #$25EF);
  add('&bigcup;', #$22C3);
  add('&bigodot;', #$2A00);
  add('&bigoplus;', #$2A01);
  add('&bigotimes;', #$2A02);
  add('&bigsqcup;', #$2A06);
  add('&bigstar;', #$2605);
  add('&bigtriangledown;', #$25BD);
  add('&bigtriangleup;', #$25B3);
  add('&biguplus;', #$2A04);
  add('&bigvee;', #$22C1);
  add('&bigwedge;', #$22C0);
  add('&bkarow;', #$290D);
  add('&blacklozenge;', #$29EB);
  add('&blacksquare;', #$25AA);
  add('&blacktriangle;', #$25B4);
  add('&blacktriangledown;', #$25BE);
  add('&blacktriangleleft;', #$25C2);
  add('&blacktriangleright;', #$25B8);
  add('&blank;', #$2423);
  add('&blk12;', #$2592);
  add('&blk14;', #$2591);
  add('&blk34;', #$2593);
  add('&block;', #$2588);
  add('&bne;', #$003D#$20E5);
  add('&bnequiv;', #$2261#$20E5);
  add('&bnot;', #$2310);
  add('&bopf;', #$D835#$DD53);
  add('&bot;', #$22A5);
  add('&bottom;', #$22A5);
  add('&bowtie;', #$22C8);
  add('&boxDL;', #$2557);
  add('&boxDR;', #$2554);
  add('&boxDl;', #$2556);
  add('&boxDr;', #$2553);
  add('&boxH;', #$2550);
  add('&boxHD;', #$2566);
  add('&boxHU;', #$2569);
  add('&boxHd;', #$2564);
  add('&boxHu;', #$2567);
  add('&boxUL;', #$255D);
  add('&boxUR;', #$255A);
  add('&boxUl;', #$255C);
  add('&boxUr;', #$2559);
  add('&boxV;', #$2551);
  add('&boxVH;', #$256C);
  add('&boxVL;', #$2563);
  add('&boxVR;', #$2560);
  add('&boxVh;', #$256B);
  add('&boxVl;', #$2562);
  add('&boxVr;', #$255F);
  add('&boxbox;', #$29C9);
  add('&boxdL;', #$2555);
  add('&boxdR;', #$2552);
  add('&boxdl;', #$2510);
  add('&boxdr;', #$250C);
  add('&boxh;', #$2500);
  add('&boxhD;', #$2565);
  add('&boxhU;', #$2568);
  add('&boxhd;', #$252C);
  add('&boxhu;', #$2534);
  add('&boxminus;', #$229F);
  add('&boxplus;', #$229E);
  add('&boxtimes;', #$22A0);
  add('&boxuL;', #$255B);
  add('&boxuR;', #$2558);
  add('&boxul;', #$2518);
  add('&boxur;', #$2514);
  add('&boxv;', #$2502);
  add('&boxvH;', #$256A);
  add('&boxvL;', #$2561);
  add('&boxvR;', #$255E);
  add('&boxvh;', #$253C);
  add('&boxvl;', #$2524);
  add('&boxvr;', #$251C);
  add('&bprime;', #$2035);
  add('&breve;', #$02D8);
  add('&brvbar;', #$00A6);
  add('&bscr;', #$D835#$DCB7);
  add('&bsemi;', #$204F);
  add('&bsim;', #$223D);
  add('&bsime;', #$22CD);
  add('&bsol;', #$005C);
  add('&bsolb;', #$29C5);
  add('&bsolhsub;', #$27C8);
  add('&bull;', #$2022);
  add('&bullet;', #$2022);
  add('&bump;', #$224E);
  add('&bumpE;', #$2AAE);
  add('&bumpe;', #$224F);
  add('&bumpeq;', #$224F);
  add('&cacute;', #$0107);
  add('&cap;', #$2229);
  add('&capand;', #$2A44);
  add('&capbrcup;', #$2A49);
  add('&capcap;', #$2A4B);
  add('&capcup;', #$2A47);
  add('&capdot;', #$2A40);
  add('&caps;', #$2229#$FE00);
  add('&caret;', #$2041);
  add('&caron;', #$02C7);
  add('&ccaps;', #$2A4D);
  add('&ccaron;', #$010D);
  add('&ccedil;', #$00E7);
  add('&ccirc;', #$0109);
  add('&ccups;', #$2A4C);
  add('&ccupssm;', #$2A50);
  add('&cdot;', #$010B);
  add('&cedil;', #$00B8);
  add('&cemptyv;', #$29B2);
  add('&cent;', #$00A2);
  add('&centerdot;', #$00B7);
  add('&cfr;', #$D835#$DD20);
  add('&chcy;', #$0447);
  add('&check;', #$2713);
  add('&checkmark;', #$2713);
  add('&chi;', #$03C7);
  add('&cir;', #$25CB);
  add('&cirE;', #$29C3);
  add('&circ;', #$02C6);
  add('&circeq;', #$2257);
  add('&circlearrowleft;', #$21BA);
  add('&circlearrowright;', #$21BB);
  add('&circledR;', #$00AE);
  add('&circledS;', #$24C8);
  add('&circledast;', #$229B);
  add('&circledcirc;', #$229A);
  add('&circleddash;', #$229D);
  add('&cire;', #$2257);
  add('&cirfnint;', #$2A10);
  add('&cirmid;', #$2AEF);
  add('&cirscir;', #$29C2);
  add('&clubs;', #$2663);
  add('&clubsuit;', #$2663);
  add('&colon;', #$003A);
  add('&colone;', #$2254);
  add('&coloneq;', #$2254);
  add('&comma;', #$002C);
  add('&commat;', #$0040);
  add('&comp;', #$2201);
  add('&compfn;', #$2218);
  add('&complement;', #$2201);
  add('&complexes;', #$2102);
  add('&cong;', #$2245);
  add('&congdot;', #$2A6D);
  add('&conint;', #$222E);
  add('&copf;', #$D835#$DD54);
  add('&coprod;', #$2210);
  add('&copy;', #$00A9);
  add('&copysr;', #$2117);
  add('&crarr;', #$21B5);
  add('&cross;', #$2717);
  add('&cscr;', #$D835#$DCB8);
  add('&csub;', #$2ACF);
  add('&csube;', #$2AD1);
  add('&csup;', #$2AD0);
  add('&csupe;', #$2AD2);
  add('&ctdot;', #$22EF);
  add('&cudarrl;', #$2938);
  add('&cudarrr;', #$2935);
  add('&cuepr;', #$22DE);
  add('&cuesc;', #$22DF);
  add('&cularr;', #$21B6);
  add('&cularrp;', #$293D);
  add('&cup;', #$222A);
  add('&cupbrcap;', #$2A48);
  add('&cupcap;', #$2A46);
  add('&cupcup;', #$2A4A);
  add('&cupdot;', #$228D);
  add('&cupor;', #$2A45);
  add('&cups;', #$222A#$FE00);
  add('&curarr;', #$21B7);
  add('&curarrm;', #$293C);
  add('&curlyeqprec;', #$22DE);
  add('&curlyeqsucc;', #$22DF);
  add('&curlyvee;', #$22CE);
  add('&curlywedge;', #$22CF);
  add('&curren;', #$00A4);
  add('&curvearrowleft;', #$21B6);
  add('&curvearrowright;', #$21B7);
  add('&cuvee;', #$22CE);
  add('&cuwed;', #$22CF);
  add('&cwconint;', #$2232);
  add('&cwint;', #$2231);
  add('&cylcty;', #$232D);
  add('&dArr;', #$21D3);
  add('&dHar;', #$2965);
  add('&dagger;', #$2020);
  add('&daleth;', #$2138);
  add('&darr;', #$2193);
  add('&dash;', #$2010);
  add('&dashv;', #$22A3);
  add('&dbkarow;', #$290F);
  add('&dblac;', #$02DD);
  add('&dcaron;', #$010F);
  add('&dcy;', #$0434);
  add('&dd;', #$2146);
  add('&ddagger;', #$2021);
  add('&ddarr;', #$21CA);
  add('&ddotseq;', #$2A77);
  add('&deg;', #$00B0);
  add('&delta;', #$03B4);
  add('&demptyv;', #$29B1);
  add('&dfisht;', #$297F);
  add('&dfr;', #$D835#$DD21);
  add('&dharl;', #$21C3);
  add('&dharr;', #$21C2);
  add('&diam;', #$22C4);
  add('&diamond;', #$22C4);
  add('&diamondsuit;', #$2666);
  add('&diams;', #$2666);
  add('&die;', #$00A8);
  add('&digamma;', #$03DD);
  add('&disin;', #$22F2);
  add('&div;', #$00F7);
  add('&divide;', #$00F7);
  add('&divideontimes;', #$22C7);
  add('&divonx;', #$22C7);
  add('&djcy;', #$0452);
  add('&dlcorn;', #$231E);
  add('&dlcrop;', #$230D);
  add('&dollar;', #$0024);
  add('&dopf;', #$D835#$DD55);
  add('&dot;', #$02D9);
  add('&doteq;', #$2250);
  add('&doteqdot;', #$2251);
  add('&dotminus;', #$2238);
  add('&dotplus;', #$2214);
  add('&dotsquare;', #$22A1);
  add('&doublebarwedge;', #$2306);
  add('&downarrow;', #$2193);
  add('&downdownarrows;', #$21CA);
  add('&downharpoonleft;', #$21C3);
  add('&downharpoonright;', #$21C2);
  add('&drbkarow;', #$2910);
  add('&drcorn;', #$231F);
  add('&drcrop;', #$230C);
  add('&dscr;', #$D835#$DCB9);
  add('&dscy;', #$0455);
  add('&dsol;', #$29F6);
  add('&dstrok;', #$0111);
  add('&dtdot;', #$22F1);
  add('&dtri;', #$25BF);
  add('&dtrif;', #$25BE);
  add('&duarr;', #$21F5);
  add('&duhar;', #$296F);
  add('&dwangle;', #$29A6);
  add('&dzcy;', #$045F);
  add('&dzigrarr;', #$27FF);
  add('&eDDot;', #$2A77);
  add('&eDot;', #$2251);
  add('&eacute;', #$00E9);
  add('&easter;', #$2A6E);
  add('&ecaron;', #$011B);
  add('&ecir;', #$2256);
  add('&ecirc;', #$00EA);
  add('&ecolon;', #$2255);
  add('&ecy;', #$044D);
  add('&edot;', #$0117);
  add('&ee;', #$2147);
  add('&efDot;', #$2252);
  add('&efr;', #$D835#$DD22);
  add('&eg;', #$2A9A);
  add('&egrave;', #$00E8);
  add('&egs;', #$2A96);
  add('&egsdot;', #$2A98);
  add('&el;', #$2A99);
  add('&elinters;', #$23E7);
  add('&ell;', #$2113);
  add('&els;', #$2A95);
  add('&elsdot;', #$2A97);
  add('&emacr;', #$0113);
  add('&empty;', #$2205);
  add('&emptyset;', #$2205);
  add('&emptyv;', #$2205);
  add('&emsp13;', #$2004);
  add('&emsp14;', #$2005);
  add('&emsp;', #$2003);
  add('&eng;', #$014B);
  add('&ensp;', #$2002);
  add('&eogon;', #$0119);
  add('&eopf;', #$D835#$DD56);
  add('&epar;', #$22D5);
  add('&eparsl;', #$29E3);
  add('&eplus;', #$2A71);
  add('&epsi;', #$03B5);
  add('&epsilon;', #$03B5);
  add('&epsiv;', #$03F5);
  add('&eqcirc;', #$2256);
  add('&eqcolon;', #$2255);
  add('&eqsim;', #$2242);
  add('&eqslantgtr;', #$2A96);
  add('&eqslantless;', #$2A95);
  add('&equals;', #$003D);
  add('&equest;', #$225F);
  add('&equiv;', #$2261);
  add('&equivDD;', #$2A78);
  add('&eqvparsl;', #$29E5);
  add('&erDot;', #$2253);
  add('&erarr;', #$2971);
  add('&escr;', #$212F);
  add('&esdot;', #$2250);
  add('&esim;', #$2242);
  add('&eta;', #$03B7);
  add('&eth;', #$00F0);
  add('&euml;', #$00EB);
  add('&euro;', #$20AC);
  add('&excl;', #$0021);
  add('&exist;', #$2203);
  add('&expectation;', #$2130);
  add('&exponentiale;', #$2147);
  add('&fallingdotseq;', #$2252);
  add('&fcy;', #$0444);
  add('&female;', #$2640);
  add('&ffilig;', #$FB03);
  add('&fflig;', #$FB00);
  add('&ffllig;', #$FB04);
  add('&ffr;', #$D835#$DD23);
  add('&filig;', #$FB01);
  add('&fjlig;', #$0066#$006A);
  add('&flat;', #$266D);
  add('&fllig;', #$FB02);
  add('&fltns;', #$25B1);
  add('&fnof;', #$0192);
  add('&fopf;', #$D835#$DD57);
  add('&forall;', #$2200);
  add('&fork;', #$22D4);
  add('&forkv;', #$2AD9);
  add('&fpartint;', #$2A0D);
  add('&frac12;', #$00BD);
  add('&frac13;', #$2153);
  add('&frac14;', #$00BC);
  add('&frac15;', #$2155);
  add('&frac16;', #$2159);
  add('&frac18;', #$215B);
  add('&frac23;', #$2154);
  add('&frac25;', #$2156);
  add('&frac34;', #$00BE);
  add('&frac35;', #$2157);
  add('&frac38;', #$215C);
  add('&frac45;', #$2158);
  add('&frac56;', #$215A);
  add('&frac58;', #$215D);
  add('&frac78;', #$215E);
  add('&frasl;', #$2044);
  add('&frown;', #$2322);
  add('&fscr;', #$D835#$DCBB);
  add('&gE;', #$2267);
  add('&gEl;', #$2A8C);
  add('&gacute;', #$01F5);
  add('&gamma;', #$03B3);
  add('&gammad;', #$03DD);
  add('&gap;', #$2A86);
  add('&gbreve;', #$011F);
  add('&gcirc;', #$011D);
  add('&gcy;', #$0433);
  add('&gdot;', #$0121);
  add('&ge;', #$2265);
  add('&gel;', #$22DB);
  add('&geq;', #$2265);
  add('&geqq;', #$2267);
  add('&geqslant;', #$2A7E);
  add('&ges;', #$2A7E);
  add('&gescc;', #$2AA9);
  add('&gesdot;', #$2A80);
  add('&gesdoto;', #$2A82);
  add('&gesdotol;', #$2A84);
  add('&gesl;', #$22DB#$FE00);
  add('&gesles;', #$2A94);
  add('&gfr;', #$D835#$DD24);
  add('&gg;', #$226B);
  add('&ggg;', #$22D9);
  add('&gimel;', #$2137);
  add('&gjcy;', #$0453);
  add('&gl;', #$2277);
  add('&glE;', #$2A92);
  add('&gla;', #$2AA5);
  add('&glj;', #$2AA4);
  add('&gnE;', #$2269);
  add('&gnap;', #$2A8A);
  add('&gnapprox;', #$2A8A);
  add('&gne;', #$2A88);
  add('&gneq;', #$2A88);
  add('&gneqq;', #$2269);
  add('&gnsim;', #$22E7);
  add('&gopf;', #$D835#$DD58);
  add('&grave;', #$0060);
  add('&gscr;', #$210A);
  add('&gsim;', #$2273);
  add('&gsime;', #$2A8E);
  add('&gsiml;', #$2A90);
  add('&gt;', #$003E);
  add('&gtcc;', #$2AA7);
  add('&gtcir;', #$2A7A);
  add('&gtdot;', #$22D7);
  add('&gtlPar;', #$2995);
  add('&gtquest;', #$2A7C);
  add('&gtrapprox;', #$2A86);
  add('&gtrarr;', #$2978);
  add('&gtrdot;', #$22D7);
  add('&gtreqless;', #$22DB);
  add('&gtreqqless;', #$2A8C);
  add('&gtrless;', #$2277);
  add('&gtrsim;', #$2273);
  add('&gvertneqq;', #$2269#$FE00);
  add('&gvnE;', #$2269#$FE00);
  add('&hArr;', #$21D4);
  add('&hairsp;', #$200A);
  add('&half;', #$00BD);
  add('&hamilt;', #$210B);
  add('&hardcy;', #$044A);
  add('&harr;', #$2194);
  add('&harrcir;', #$2948);
  add('&harrw;', #$21AD);
  add('&hbar;', #$210F);
  add('&hcirc;', #$0125);
  add('&hearts;', #$2665);
  add('&heartsuit;', #$2665);
  add('&hellip;', #$2026);
  add('&hercon;', #$22B9);
  add('&hfr;', #$D835#$DD25);
  add('&hksearow;', #$2925);
  add('&hkswarow;', #$2926);
  add('&hoarr;', #$21FF);
  add('&homtht;', #$223B);
  add('&hookleftarrow;', #$21A9);
  add('&hookrightarrow;', #$21AA);
  add('&hopf;', #$D835#$DD59);
  add('&horbar;', #$2015);
  add('&hscr;', #$D835#$DCBD);
  add('&hslash;', #$210F);
  add('&hstrok;', #$0127);
  add('&hybull;', #$2043);
  add('&hyphen;', #$2010);
  add('&iacute;', #$00ED);
  add('&ic;', #$2063);
  add('&icirc;', #$00EE);
  add('&icy;', #$0438);
  add('&iecy;', #$0435);
  add('&iexcl;', #$00A1);
  add('&iff;', #$21D4);
  add('&ifr;', #$D835#$DD26);
  add('&igrave;', #$00EC);
  add('&ii;', #$2148);
  add('&iiiint;', #$2A0C);
  add('&iiint;', #$222D);
  add('&iinfin;', #$29DC);
  add('&iiota;', #$2129);
  add('&ijlig;', #$0133);
  add('&imacr;', #$012B);
  add('&image;', #$2111);
  add('&imagline;', #$2110);
  add('&imagpart;', #$2111);
  add('&imath;', #$0131);
  add('&imof;', #$22B7);
  add('&imped;', #$01B5);
  add('&in;', #$2208);
  add('&incare;', #$2105);
  add('&infin;', #$221E);
  add('&infintie;', #$29DD);
  add('&inodot;', #$0131);
  add('&int;', #$222B);
  add('&intcal;', #$22BA);
  add('&integers;', #$2124);
  add('&intercal;', #$22BA);
  add('&intlarhk;', #$2A17);
  add('&intprod;', #$2A3C);
  add('&iocy;', #$0451);
  add('&iogon;', #$012F);
  add('&iopf;', #$D835#$DD5A);
  add('&iota;', #$03B9);
  add('&iprod;', #$2A3C);
  add('&iquest;', #$00BF);
  add('&iscr;', #$D835#$DCBE);
  add('&isin;', #$2208);
  add('&isinE;', #$22F9);
  add('&isindot;', #$22F5);
  add('&isins;', #$22F4);
  add('&isinsv;', #$22F3);
  add('&isinv;', #$2208);
  add('&it;', #$2062);
  add('&itilde;', #$0129);
  add('&iukcy;', #$0456);
  add('&iuml;', #$00EF);
  add('&jcirc;', #$0135);
  add('&jcy;', #$0439);
  add('&jfr;', #$D835#$DD27);
  add('&jmath;', #$0237);
  add('&jopf;', #$D835#$DD5B);
  add('&jscr;', #$D835#$DCBF);
  add('&jsercy;', #$0458);
  add('&jukcy;', #$0454);
  add('&kappa;', #$03BA);
  add('&kappav;', #$03F0);
  add('&kcedil;', #$0137);
  add('&kcy;', #$043A);
  add('&kfr;', #$D835#$DD28);
  add('&kgreen;', #$0138);
  add('&khcy;', #$0445);
  add('&kjcy;', #$045C);
  add('&kopf;', #$D835#$DD5C);
  add('&kscr;', #$D835#$DCC0);
  add('&lAarr;', #$21DA);
  add('&lArr;', #$21D0);
  add('&lAtail;', #$291B);
  add('&lBarr;', #$290E);
  add('&lE;', #$2266);
  add('&lEg;', #$2A8B);
  add('&lHar;', #$2962);
  add('&lacute;', #$013A);
  add('&laemptyv;', #$29B4);
  add('&lagran;', #$2112);
  add('&lambda;', #$03BB);
  add('&lang;', #$27E8);
  add('&langd;', #$2991);
  add('&langle;', #$27E8);
  add('&lap;', #$2A85);
  add('&laquo;', #$00AB);
  add('&larr;', #$2190);
  add('&larrb;', #$21E4);
  add('&larrbfs;', #$291F);
  add('&larrfs;', #$291D);
  add('&larrhk;', #$21A9);
  add('&larrlp;', #$21AB);
  add('&larrpl;', #$2939);
  add('&larrsim;', #$2973);
  add('&larrtl;', #$21A2);
  add('&lat;', #$2AAB);
  add('&latail;', #$2919);
  add('&late;', #$2AAD);
  add('&lates;', #$2AAD#$FE00);
  add('&lbarr;', #$290C);
  add('&lbbrk;', #$2772);
  add('&lbrace;', #$007B);
  add('&lbrack;', #$005B);
  add('&lbrke;', #$298B);
  add('&lbrksld;', #$298F);
  add('&lbrkslu;', #$298D);
  add('&lcaron;', #$013E);
  add('&lcedil;', #$013C);
  add('&lceil;', #$2308);
  add('&lcub;', #$007B);
  add('&lcy;', #$043B);
  add('&ldca;', #$2936);
  add('&ldquo;', #$201C);
  add('&ldquor;', #$201E);
  add('&ldrdhar;', #$2967);
  add('&ldrushar;', #$294B);
  add('&ldsh;', #$21B2);
  add('&le;', #$2264);
  add('&leftarrow;', #$2190);
  add('&leftarrowtail;', #$21A2);
  add('&leftharpoondown;', #$21BD);
  add('&leftharpoonup;', #$21BC);
  add('&leftleftarrows;', #$21C7);
  add('&leftrightarrow;', #$2194);
  add('&leftrightarrows;', #$21C6);
  add('&leftrightharpoons;', #$21CB);
  add('&leftrightsquigarrow;', #$21AD);
  add('&leftthreetimes;', #$22CB);
  add('&leg;', #$22DA);
  add('&leq;', #$2264);
  add('&leqq;', #$2266);
  add('&leqslant;', #$2A7D);
  add('&les;', #$2A7D);
  add('&lescc;', #$2AA8);
  add('&lesdot;', #$2A7F);
  add('&lesdoto;', #$2A81);
  add('&lesdotor;', #$2A83);
  add('&lesg;', #$22DA#$FE00);
  add('&lesges;', #$2A93);
  add('&lessapprox;', #$2A85);
  add('&lessdot;', #$22D6);
  add('&lesseqgtr;', #$22DA);
  add('&lesseqqgtr;', #$2A8B);
  add('&lessgtr;', #$2276);
  add('&lesssim;', #$2272);
  add('&lfisht;', #$297C);
  add('&lfloor;', #$230A);
  add('&lfr;', #$D835#$DD29);
  add('&lg;', #$2276);
  add('&lgE;', #$2A91);
  add('&lhard;', #$21BD);
  add('&lharu;', #$21BC);
  add('&lharul;', #$296A);
  add('&lhblk;', #$2584);
  add('&ljcy;', #$0459);
  add('&ll;', #$226A);
  add('&llarr;', #$21C7);
  add('&llcorner;', #$231E);
  add('&llhard;', #$296B);
  add('&lltri;', #$25FA);
  add('&lmidot;', #$0140);
  add('&lmoust;', #$23B0);
  add('&lmoustache;', #$23B0);
  add('&lnE;', #$2268);
  add('&lnap;', #$2A89);
  add('&lnapprox;', #$2A89);
  add('&lne;', #$2A87);
  add('&lneq;', #$2A87);
  add('&lneqq;', #$2268);
  add('&lnsim;', #$22E6);
  add('&loang;', #$27EC);
  add('&loarr;', #$21FD);
  add('&lobrk;', #$27E6);
  add('&longleftarrow;', #$27F5);
  add('&longleftrightarrow;', #$27F7);
  add('&longmapsto;', #$27FC);
  add('&longrightarrow;', #$27F6);
  add('&looparrowleft;', #$21AB);
  add('&looparrowright;', #$21AC);
  add('&lopar;', #$2985);
  add('&lopf;', #$D835#$DD5D);
  add('&loplus;', #$2A2D);
  add('&lotimes;', #$2A34);
  add('&lowast;', #$2217);
  add('&lowbar;', #$005F);
  add('&loz;', #$25CA);
  add('&lozenge;', #$25CA);
  add('&lozf;', #$29EB);
  add('&lpar;', #$0028);
  add('&lparlt;', #$2993);
  add('&lrarr;', #$21C6);
  add('&lrcorner;', #$231F);
  add('&lrhar;', #$21CB);
  add('&lrhard;', #$296D);
  add('&lrm;', #$200E);
  add('&lrtri;', #$22BF);
  add('&lsaquo;', #$2039);
  add('&lscr;', #$D835#$DCC1);
  add('&lsh;', #$21B0);
  add('&lsim;', #$2272);
  add('&lsime;', #$2A8D);
  add('&lsimg;', #$2A8F);
  add('&lsqb;', #$005B);
  add('&lsquo;', #$2018);
  add('&lsquor;', #$201A);
  add('&lstrok;', #$0142);
  add('&lt;', #$003C);
  add('&ltcc;', #$2AA6);
  add('&ltcir;', #$2A79);
  add('&ltdot;', #$22D6);
  add('&lthree;', #$22CB);
  add('&ltimes;', #$22C9);
  add('&ltlarr;', #$2976);
  add('&ltquest;', #$2A7B);
  add('&ltrPar;', #$2996);
  add('&ltri;', #$25C3);
  add('&ltrie;', #$22B4);
  add('&ltrif;', #$25C2);
  add('&lurdshar;', #$294A);
  add('&luruhar;', #$2966);
  add('&lvertneqq;', #$2268#$FE00);
  add('&lvnE;', #$2268#$FE00);
  add('&mDDot;', #$223A);
  add('&macr;', #$00AF);
  add('&male;', #$2642);
  add('&malt;', #$2720);
  add('&maltese;', #$2720);
  add('&map;', #$21A6);
  add('&mapsto;', #$21A6);
  add('&mapstodown;', #$21A7);
  add('&mapstoleft;', #$21A4);
  add('&mapstoup;', #$21A5);
  add('&marker;', #$25AE);
  add('&mcomma;', #$2A29);
  add('&mcy;', #$043C);
  add('&mdash;', #$2014);
  add('&measuredangle;', #$2221);
  add('&mfr;', #$D835#$DD2A);
  add('&mho;', #$2127);
  add('&micro;', #$00B5);
  add('&mid;', #$2223);
  add('&midast;', #$002A);
  add('&midcir;', #$2AF0);
  add('&middot;', #$00B7);
  add('&minus;', #$2212);
  add('&minusb;', #$229F);
  add('&minusd;', #$2238);
  add('&minusdu;', #$2A2A);
  add('&mlcp;', #$2ADB);
  add('&mldr;', #$2026);
  add('&mnplus;', #$2213);
  add('&models;', #$22A7);
  add('&mopf;', #$D835#$DD5E);
  add('&mp;', #$2213);
  add('&mscr;', #$D835#$DCC2);
  add('&mstpos;', #$223E);
  add('&mu;', #$03BC);
  add('&multimap;', #$22B8);
  add('&mumap;', #$22B8);
  add('&nGg;', #$22D9#$0338);
  add('&nGt;', #$226B#$20D2);
  add('&nGtv;', #$226B#$0338);
  add('&nLeftarrow;', #$21CD);
  add('&nLeftrightarrow;', #$21CE);
  add('&nLl;', #$22D8#$0338);
  add('&nLt;', #$226A#$20D2);
  add('&nLtv;', #$226A#$0338);
  add('&nRightarrow;', #$21CF);
  add('&nVDash;', #$22AF);
  add('&nVdash;', #$22AE);
  add('&nabla;', #$2207);
  add('&nacute;', #$0144);
  add('&nang;', #$2220#$20D2);
  add('&nap;', #$2249);
  add('&napE;', #$2A70#$0338);
  add('&napid;', #$224B#$0338);
  add('&napos;', #$0149);
  add('&napprox;', #$2249);
  add('&natur;', #$266E);
  add('&natural;', #$266E);
  add('&naturals;', #$2115);
  add('&nbsp;', #$00A0);
  add('&nbump;', #$224E#$0338);
  add('&nbumpe;', #$224F#$0338);
  add('&ncap;', #$2A43);
  add('&ncaron;', #$0148);
  add('&ncedil;', #$0146);
  add('&ncong;', #$2247);
  add('&ncongdot;', #$2A6D#$0338);
  add('&ncup;', #$2A42);
  add('&ncy;', #$043D);
  add('&ndash;', #$2013);
  add('&ne;', #$2260);
  add('&neArr;', #$21D7);
  add('&nearhk;', #$2924);
  add('&nearr;', #$2197);
  add('&nearrow;', #$2197);
  add('&nedot;', #$2250#$0338);
  add('&nequiv;', #$2262);
  add('&nesear;', #$2928);
  add('&nesim;', #$2242#$0338);
  add('&nexist;', #$2204);
  add('&nexists;', #$2204);
  add('&nfr;', #$D835#$DD2B);
  add('&ngE;', #$2267#$0338);
  add('&nge;', #$2271);
  add('&ngeq;', #$2271);
  add('&ngeqq;', #$2267#$0338);
  add('&ngeqslant;', #$2A7E#$0338);
  add('&nges;', #$2A7E#$0338);
  add('&ngsim;', #$2275);
  add('&ngt;', #$226F);
  add('&ngtr;', #$226F);
  add('&nhArr;', #$21CE);
  add('&nharr;', #$21AE);
  add('&nhpar;', #$2AF2);
  add('&ni;', #$220B);
  add('&nis;', #$22FC);
  add('&nisd;', #$22FA);
  add('&niv;', #$220B);
  add('&njcy;', #$045A);
  add('&nlArr;', #$21CD);
  add('&nlE;', #$2266#$0338);
  add('&nlarr;', #$219A);
  add('&nldr;', #$2025);
  add('&nle;', #$2270);
  add('&nleftarrow;', #$219A);
  add('&nleftrightarrow;', #$21AE);
  add('&nleq;', #$2270);
  add('&nleqq;', #$2266#$0338);
  add('&nleqslant;', #$2A7D#$0338);
  add('&nles;', #$2A7D#$0338);
  add('&nless;', #$226E);
  add('&nlsim;', #$2274);
  add('&nlt;', #$226E);
  add('&nltri;', #$22EA);
  add('&nltrie;', #$22EC);
  add('&nmid;', #$2224);
  add('&nopf;', #$D835#$DD5F);
  add('&not;', #$00AC);
  add('&notin;', #$2209);
  add('&notinE;', #$22F9#$0338);
  add('&notindot;', #$22F5#$0338);
  add('&notinva;', #$2209);
  add('&notinvb;', #$22F7);
  add('&notinvc;', #$22F6);
  add('&notni;', #$220C);
  add('&notniva;', #$220C);
  add('&notnivb;', #$22FE);
  add('&notnivc;', #$22FD);
  add('&npar;', #$2226);
  add('&nparallel;', #$2226);
  add('&nparsl;', #$2AFD#$20E5);
  add('&npart;', #$2202#$0338);
  add('&npolint;', #$2A14);
  add('&npr;', #$2280);
  add('&nprcue;', #$22E0);
  add('&npre;', #$2AAF#$0338);
  add('&nprec;', #$2280);
  add('&npreceq;', #$2AAF#$0338);
  add('&nrArr;', #$21CF);
  add('&nrarr;', #$219B);
  add('&nrarrc;', #$2933#$0338);
  add('&nrarrw;', #$219D#$0338);
  add('&nrightarrow;', #$219B);
  add('&nrtri;', #$22EB);
  add('&nrtrie;', #$22ED);
  add('&nsc;', #$2281);
  add('&nsccue;', #$22E1);
  add('&nsce;', #$2AB0#$0338);
  add('&nscr;', #$D835#$DCC3);
  add('&nshortmid;', #$2224);
  add('&nshortparallel;', #$2226);
  add('&nsim;', #$2241);
  add('&nsime;', #$2244);
  add('&nsimeq;', #$2244);
  add('&nsmid;', #$2224);
  add('&nspar;', #$2226);
  add('&nsqsube;', #$22E2);
  add('&nsqsupe;', #$22E3);
  add('&nsub;', #$2284);
  add('&nsubE;', #$2AC5#$0338);
  add('&nsube;', #$2288);
  add('&nsubset;', #$2282#$20D2);
  add('&nsubseteq;', #$2288);
  add('&nsubseteqq;', #$2AC5#$0338);
  add('&nsucc;', #$2281);
  add('&nsucceq;', #$2AB0#$0338);
  add('&nsup;', #$2285);
  add('&nsupE;', #$2AC6#$0338);
  add('&nsupe;', #$2289);
  add('&nsupset;', #$2283#$20D2);
  add('&nsupseteq;', #$2289);
  add('&nsupseteqq;', #$2AC6#$0338);
  add('&ntgl;', #$2279);
  add('&ntilde;', #$00F1);
  add('&ntlg;', #$2278);
  add('&ntriangleleft;', #$22EA);
  add('&ntrianglelefteq;', #$22EC);
  add('&ntriangleright;', #$22EB);
  add('&ntrianglerighteq;', #$22ED);
  add('&nu;', #$03BD);
  add('&num;', #$0023);
  add('&numero;', #$2116);
  add('&numsp;', #$2007);
  add('&nvDash;', #$22AD);
  add('&nvHarr;', #$2904);
  add('&nvap;', #$224D#$20D2);
  add('&nvdash;', #$22AC);
  add('&nvge;', #$2265#$20D2);
  add('&nvgt;', #$003E#$20D2);
  add('&nvinfin;', #$29DE);
  add('&nvlArr;', #$2902);
  add('&nvle;', #$2264#$20D2);
  add('&nvlt;', #$003C#$20D2);
  add('&nvltrie;', #$22B4#$20D2);
  add('&nvrArr;', #$2903);
  add('&nvrtrie;', #$22B5#$20D2);
  add('&nvsim;', #$223C#$20D2);
  add('&nwArr;', #$21D6);
  add('&nwarhk;', #$2923);
  add('&nwarr;', #$2196);
  add('&nwarrow;', #$2196);
  add('&nwnear;', #$2927);
  add('&oS;', #$24C8);
  add('&oacute;', #$00F3);
  add('&oast;', #$229B);
  add('&ocir;', #$229A);
  add('&ocirc;', #$00F4);
  add('&ocy;', #$043E);
  add('&odash;', #$229D);
  add('&odblac;', #$0151);
  add('&odiv;', #$2A38);
  add('&odot;', #$2299);
  add('&odsold;', #$29BC);
  add('&oelig;', #$0153);
  add('&ofcir;', #$29BF);
  add('&ofr;', #$D835#$DD2C);
  add('&ogon;', #$02DB);
  add('&ograve;', #$00F2);
  add('&ogt;', #$29C1);
  add('&ohbar;', #$29B5);
  add('&ohm;', #$03A9);
  add('&oint;', #$222E);
  add('&olarr;', #$21BA);
  add('&olcir;', #$29BE);
  add('&olcross;', #$29BB);
  add('&oline;', #$203E);
  add('&olt;', #$29C0);
  add('&omacr;', #$014D);
  add('&omega;', #$03C9);
  add('&omicron;', #$03BF);
  add('&omid;', #$29B6);
  add('&ominus;', #$2296);
  add('&oopf;', #$D835#$DD60);
  add('&opar;', #$29B7);
  add('&operp;', #$29B9);
  add('&oplus;', #$2295);
  add('&or;', #$2228);
  add('&orarr;', #$21BB);
  add('&ord;', #$2A5D);
  add('&order;', #$2134);
  add('&orderof;', #$2134);
  add('&ordf;', #$00AA);
  add('&ordm;', #$00BA);
  add('&origof;', #$22B6);
  add('&oror;', #$2A56);
  add('&orslope;', #$2A57);
  add('&orv;', #$2A5B);
  add('&oscr;', #$2134);
  add('&oslash;', #$00F8);
  add('&osol;', #$2298);
  add('&otilde;', #$00F5);
  add('&otimes;', #$2297);
  add('&otimesas;', #$2A36);
  add('&ouml;', #$00F6);
  add('&ovbar;', #$233D);
  add('&par;', #$2225);
  add('&para;', #$00B6);
  add('&parallel;', #$2225);
  add('&parsim;', #$2AF3);
  add('&parsl;', #$2AFD);
  add('&part;', #$2202);
  add('&pcy;', #$043F);
  add('&percnt;', #$0025);
  add('&period;', #$002E);
  add('&permil;', #$2030);
  add('&perp;', #$22A5);
  add('&pertenk;', #$2031);
  add('&pfr;', #$D835#$DD2D);
  add('&phi;', #$03C6);
  add('&phiv;', #$03D5);
  add('&phmmat;', #$2133);
  add('&phone;', #$260E);
  add('&pi;', #$03C0);
  add('&pitchfork;', #$22D4);
  add('&piv;', #$03D6);
  add('&planck;', #$210F);
  add('&planckh;', #$210E);
  add('&plankv;', #$210F);
  add('&plus;', #$002B);
  add('&plusacir;', #$2A23);
  add('&plusb;', #$229E);
  add('&pluscir;', #$2A22);
  add('&plusdo;', #$2214);
  add('&plusdu;', #$2A25);
  add('&pluse;', #$2A72);
  add('&plusmn;', #$00B1);
  add('&plussim;', #$2A26);
  add('&plustwo;', #$2A27);
  add('&pm;', #$00B1);
  add('&pointint;', #$2A15);
  add('&popf;', #$D835#$DD61);
  add('&pound;', #$00A3);
  add('&pr;', #$227A);
  add('&prE;', #$2AB3);
  add('&prap;', #$2AB7);
  add('&prcue;', #$227C);
  add('&pre;', #$2AAF);
  add('&prec;', #$227A);
  add('&precapprox;', #$2AB7);
  add('&preccurlyeq;', #$227C);
  add('&preceq;', #$2AAF);
  add('&precnapprox;', #$2AB9);
  add('&precneqq;', #$2AB5);
  add('&precnsim;', #$22E8);
  add('&precsim;', #$227E);
  add('&prime;', #$2032);
  add('&primes;', #$2119);
  add('&prnE;', #$2AB5);
  add('&prnap;', #$2AB9);
  add('&prnsim;', #$22E8);
  add('&prod;', #$220F);
  add('&profalar;', #$232E);
  add('&profline;', #$2312);
  add('&profsurf;', #$2313);
  add('&prop;', #$221D);
  add('&propto;', #$221D);
  add('&prsim;', #$227E);
  add('&prurel;', #$22B0);
  add('&pscr;', #$D835#$DCC5);
  add('&psi;', #$03C8);
  add('&puncsp;', #$2008);
  add('&qfr;', #$D835#$DD2E);
  add('&qint;', #$2A0C);
  add('&qopf;', #$D835#$DD62);
  add('&qprime;', #$2057);
  add('&qscr;', #$D835#$DCC6);
  add('&quaternions;', #$210D);
  add('&quatint;', #$2A16);
  add('&quest;', #$003F);
  add('&questeq;', #$225F);
  add('&quot;', #$0022);
  add('&rAarr;', #$21DB);
  add('&rArr;', #$21D2);
  add('&rAtail;', #$291C);
  add('&rBarr;', #$290F);
  add('&rHar;', #$2964);
  add('&race;', #$223D#$0331);
  add('&racute;', #$0155);
  add('&radic;', #$221A);
  add('&raemptyv;', #$29B3);
  add('&rang;', #$27E9);
  add('&rangd;', #$2992);
  add('&range;', #$29A5);
  add('&rangle;', #$27E9);
  add('&raquo;', #$00BB);
  add('&rarr;', #$2192);
  add('&rarrap;', #$2975);
  add('&rarrb;', #$21E5);
  add('&rarrbfs;', #$2920);
  add('&rarrc;', #$2933);
  add('&rarrfs;', #$291E);
  add('&rarrhk;', #$21AA);
  add('&rarrlp;', #$21AC);
  add('&rarrpl;', #$2945);
  add('&rarrsim;', #$2974);
  add('&rarrtl;', #$21A3);
  add('&rarrw;', #$219D);
  add('&ratail;', #$291A);
  add('&ratio;', #$2236);
  add('&rationals;', #$211A);
  add('&rbarr;', #$290D);
  add('&rbbrk;', #$2773);
  add('&rbrace;', #$007D);
  add('&rbrack;', #$005D);
  add('&rbrke;', #$298C);
  add('&rbrksld;', #$298E);
  add('&rbrkslu;', #$2990);
  add('&rcaron;', #$0159);
  add('&rcedil;', #$0157);
  add('&rceil;', #$2309);
  add('&rcub;', #$007D);
  add('&rcy;', #$0440);
  add('&rdca;', #$2937);
  add('&rdldhar;', #$2969);
  add('&rdquo;', #$201D);
  add('&rdquor;', #$201D);
  add('&rdsh;', #$21B3);
  add('&real;', #$211C);
  add('&realine;', #$211B);
  add('&realpart;', #$211C);
  add('&reals;', #$211D);
  add('&rect;', #$25AD);
  add('&reg;', #$00AE);
  add('&rfisht;', #$297D);
  add('&rfloor;', #$230B);
  add('&rfr;', #$D835#$DD2F);
  add('&rhard;', #$21C1);
  add('&rharu;', #$21C0);
  add('&rharul;', #$296C);
  add('&rho;', #$03C1);
  add('&rhov;', #$03F1);
  add('&rightarrow;', #$2192);
  add('&rightarrowtail;', #$21A3);
  add('&rightharpoondown;', #$21C1);
  add('&rightharpoonup;', #$21C0);
  add('&rightleftarrows;', #$21C4);
  add('&rightleftharpoons;', #$21CC);
  add('&rightrightarrows;', #$21C9);
  add('&rightsquigarrow;', #$219D);
  add('&rightthreetimes;', #$22CC);
  add('&ring;', #$02DA);
  add('&risingdotseq;', #$2253);
  add('&rlarr;', #$21C4);
  add('&rlhar;', #$21CC);
  add('&rlm;', #$200F);
  add('&rmoust;', #$23B1);
  add('&rmoustache;', #$23B1);
  add('&rnmid;', #$2AEE);
  add('&roang;', #$27ED);
  add('&roarr;', #$21FE);
  add('&robrk;', #$27E7);
  add('&ropar;', #$2986);
  add('&ropf;', #$D835#$DD63);
  add('&roplus;', #$2A2E);
  add('&rotimes;', #$2A35);
  add('&rpar;', #$0029);
  add('&rpargt;', #$2994);
  add('&rppolint;', #$2A12);
  add('&rrarr;', #$21C9);
  add('&rsaquo;', #$203A);
  add('&rscr;', #$D835#$DCC7);
  add('&rsh;', #$21B1);
  add('&rsqb;', #$005D);
  add('&rsquo;', #$2019);
  add('&rsquor;', #$2019);
  add('&rthree;', #$22CC);
  add('&rtimes;', #$22CA);
  add('&rtri;', #$25B9);
  add('&rtrie;', #$22B5);
  add('&rtrif;', #$25B8);
  add('&rtriltri;', #$29CE);
  add('&ruluhar;', #$2968);
  add('&rx;', #$211E);
  add('&sacute;', #$015B);
  add('&sbquo;', #$201A);
  add('&sc;', #$227B);
  add('&scE;', #$2AB4);
  add('&scap;', #$2AB8);
  add('&scaron;', #$0161);
  add('&sccue;', #$227D);
  add('&sce;', #$2AB0);
  add('&scedil;', #$015F);
  add('&scirc;', #$015D);
  add('&scnE;', #$2AB6);
  add('&scnap;', #$2ABA);
  add('&scnsim;', #$22E9);
  add('&scpolint;', #$2A13);
  add('&scsim;', #$227F);
  add('&scy;', #$0441);
  add('&sdot;', #$22C5);
  add('&sdotb;', #$22A1);
  add('&sdote;', #$2A66);
  add('&seArr;', #$21D8);
  add('&searhk;', #$2925);
  add('&searr;', #$2198);
  add('&searrow;', #$2198);
  add('&sect;', #$00A7);
  add('&semi;', #$003B);
  add('&seswar;', #$2929);
  add('&setminus;', #$2216);
  add('&setmn;', #$2216);
  add('&sext;', #$2736);
  add('&sfr;', #$D835#$DD30);
  add('&sfrown;', #$2322);
  add('&sharp;', #$266F);
  add('&shchcy;', #$0449);
  add('&shcy;', #$0448);
  add('&shortmid;', #$2223);
  add('&shortparallel;', #$2225);
  add('&shy;', #$00AD);
  add('&sigma;', #$03C3);
  add('&sigmaf;', #$03C2);
  add('&sigmav;', #$03C2);
  add('&sim;', #$223C);
  add('&simdot;', #$2A6A);
  add('&sime;', #$2243);
  add('&simeq;', #$2243);
  add('&simg;', #$2A9E);
  add('&simgE;', #$2AA0);
  add('&siml;', #$2A9D);
  add('&simlE;', #$2A9F);
  add('&simne;', #$2246);
  add('&simplus;', #$2A24);
  add('&simrarr;', #$2972);
  add('&slarr;', #$2190);
  add('&smallsetminus;', #$2216);
  add('&smashp;', #$2A33);
  add('&smeparsl;', #$29E4);
  add('&smid;', #$2223);
  add('&smile;', #$2323);
  add('&smt;', #$2AAA);
  add('&smte;', #$2AAC);
  add('&smtes;', #$2AAC#$FE00);
  add('&softcy;', #$044C);
  add('&sol;', #$002F);
  add('&solb;', #$29C4);
  add('&solbar;', #$233F);
  add('&sopf;', #$D835#$DD64);
  add('&spades;', #$2660);
  add('&spadesuit;', #$2660);
  add('&spar;', #$2225);
  add('&sqcap;', #$2293);
  add('&sqcaps;', #$2293#$FE00);
  add('&sqcup;', #$2294);
  add('&sqcups;', #$2294#$FE00);
  add('&sqsub;', #$228F);
  add('&sqsube;', #$2291);
  add('&sqsubset;', #$228F);
  add('&sqsubseteq;', #$2291);
  add('&sqsup;', #$2290);
  add('&sqsupe;', #$2292);
  add('&sqsupset;', #$2290);
  add('&sqsupseteq;', #$2292);
  add('&squ;', #$25A1);
  add('&square;', #$25A1);
  add('&squarf;', #$25AA);
  add('&squf;', #$25AA);
  add('&srarr;', #$2192);
  add('&sscr;', #$D835#$DCC8);
  add('&ssetmn;', #$2216);
  add('&ssmile;', #$2323);
  add('&sstarf;', #$22C6);
  add('&star;', #$2606);
  add('&starf;', #$2605);
  add('&straightepsilon;', #$03F5);
  add('&straightphi;', #$03D5);
  add('&strns;', #$00AF);
  add('&sub;', #$2282);
  add('&subE;', #$2AC5);
  add('&subdot;', #$2ABD);
  add('&sube;', #$2286);
  add('&subedot;', #$2AC3);
  add('&submult;', #$2AC1);
  add('&subnE;', #$2ACB);
  add('&subne;', #$228A);
  add('&subplus;', #$2ABF);
  add('&subrarr;', #$2979);
  add('&subset;', #$2282);
  add('&subseteq;', #$2286);
  add('&subseteqq;', #$2AC5);
  add('&subsetneq;', #$228A);
  add('&subsetneqq;', #$2ACB);
  add('&subsim;', #$2AC7);
  add('&subsub;', #$2AD5);
  add('&subsup;', #$2AD3);
  add('&succ;', #$227B);
  add('&succapprox;', #$2AB8);
  add('&succcurlyeq;', #$227D);
  add('&succeq;', #$2AB0);
  add('&succnapprox;', #$2ABA);
  add('&succneqq;', #$2AB6);
  add('&succnsim;', #$22E9);
  add('&succsim;', #$227F);
  add('&sum;', #$2211);
  add('&sung;', #$266A);
  add('&sup1;', #$00B9);
  add('&sup2;', #$00B2);
  add('&sup3;', #$00B3);
  add('&sup;', #$2283);
  add('&supE;', #$2AC6);
  add('&supdot;', #$2ABE);
  add('&supdsub;', #$2AD8);
  add('&supe;', #$2287);
  add('&supedot;', #$2AC4);
  add('&suphsol;', #$27C9);
  add('&suphsub;', #$2AD7);
  add('&suplarr;', #$297B);
  add('&supmult;', #$2AC2);
  add('&supnE;', #$2ACC);
  add('&supne;', #$228B);
  add('&supplus;', #$2AC0);
  add('&supset;', #$2283);
  add('&supseteq;', #$2287);
  add('&supseteqq;', #$2AC6);
  add('&supsetneq;', #$228B);
  add('&supsetneqq;', #$2ACC);
  add('&supsim;', #$2AC8);
  add('&supsub;', #$2AD4);
  add('&supsup;', #$2AD6);
  add('&swArr;', #$21D9);
  add('&swarhk;', #$2926);
  add('&swarr;', #$2199);
  add('&swarrow;', #$2199);
  add('&swnwar;', #$292A);
  add('&szlig;', #$00DF);
  add('&target;', #$2316);
  add('&tau;', #$03C4);
  add('&tbrk;', #$23B4);
  add('&tcaron;', #$0165);
  add('&tcedil;', #$0163);
  add('&tcy;', #$0442);
  add('&tdot;', #$20DB);
  add('&telrec;', #$2315);
  add('&tfr;', #$D835#$DD31);
  add('&there4;', #$2234);
  add('&therefore;', #$2234);
  add('&theta;', #$03B8);
  add('&thetasym;', #$03D1);
  add('&thetav;', #$03D1);
  add('&thickapprox;', #$2248);
  add('&thicksim;', #$223C);
  add('&thinsp;', #$2009);
  add('&thkap;', #$2248);
  add('&thksim;', #$223C);
  add('&thorn;', #$00FE);
  add('&tilde;', #$02DC);
  add('&times;', #$00D7);
  add('&timesb;', #$22A0);
  add('&timesbar;', #$2A31);
  add('&timesd;', #$2A30);
  add('&tint;', #$222D);
  add('&toea;', #$2928);
  add('&top;', #$22A4);
  add('&topbot;', #$2336);
  add('&topcir;', #$2AF1);
  add('&topf;', #$D835#$DD65);
  add('&topfork;', #$2ADA);
  add('&tosa;', #$2929);
  add('&tprime;', #$2034);
  add('&trade;', #$2122);
  add('&triangle;', #$25B5);
  add('&triangledown;', #$25BF);
  add('&triangleleft;', #$25C3);
  add('&trianglelefteq;', #$22B4);
  add('&triangleq;', #$225C);
  add('&triangleright;', #$25B9);
  add('&trianglerighteq;', #$22B5);
  add('&tridot;', #$25EC);
  add('&trie;', #$225C);
  add('&triminus;', #$2A3A);
  add('&triplus;', #$2A39);
  add('&trisb;', #$29CD);
  add('&tritime;', #$2A3B);
  add('&trpezium;', #$23E2);
  add('&tscr;', #$D835#$DCC9);
  add('&tscy;', #$0446);
  add('&tshcy;', #$045B);
  add('&tstrok;', #$0167);
  add('&twixt;', #$226C);
  add('&twoheadleftarrow;', #$219E);
  add('&twoheadrightarrow;', #$21A0);
  add('&uArr;', #$21D1);
  add('&uHar;', #$2963);
  add('&uacute;', #$00FA);
  add('&uarr;', #$2191);
  add('&ubrcy;', #$045E);
  add('&ubreve;', #$016D);
  add('&ucirc;', #$00FB);
  add('&ucy;', #$0443);
  add('&udarr;', #$21C5);
  add('&udblac;', #$0171);
  add('&udhar;', #$296E);
  add('&ufisht;', #$297E);
  add('&ufr;', #$D835#$DD32);
  add('&ugrave;', #$00F9);
  add('&uharl;', #$21BF);
  add('&uharr;', #$21BE);
  add('&uhblk;', #$2580);
  add('&ulcorn;', #$231C);
  add('&ulcorner;', #$231C);
  add('&ulcrop;', #$230F);
  add('&ultri;', #$25F8);
  add('&umacr;', #$016B);
  add('&uml;', #$00A8);
  add('&uogon;', #$0173);
  add('&uopf;', #$D835#$DD66);
  add('&uparrow;', #$2191);
  add('&updownarrow;', #$2195);
  add('&upharpoonleft;', #$21BF);
  add('&upharpoonright;', #$21BE);
  add('&uplus;', #$228E);
  add('&upsi;', #$03C5);
  add('&upsih;', #$03D2);
  add('&upsilon;', #$03C5);
  add('&upuparrows;', #$21C8);
  add('&urcorn;', #$231D);
  add('&urcorner;', #$231D);
  add('&urcrop;', #$230E);
  add('&uring;', #$016F);
  add('&urtri;', #$25F9);
  add('&uscr;', #$D835#$DCCA);
  add('&utdot;', #$22F0);
  add('&utilde;', #$0169);
  add('&utri;', #$25B5);
  add('&utrif;', #$25B4);
  add('&uuarr;', #$21C8);
  add('&uuml;', #$00FC);
  add('&uwangle;', #$29A7);
  add('&vArr;', #$21D5);
  add('&vBar;', #$2AE8);
  add('&vBarv;', #$2AE9);
  add('&vDash;', #$22A8);
  add('&vangrt;', #$299C);
  add('&varepsilon;', #$03F5);
  add('&varkappa;', #$03F0);
  add('&varnothing;', #$2205);
  add('&varphi;', #$03D5);
  add('&varpi;', #$03D6);
  add('&varpropto;', #$221D);
  add('&varr;', #$2195);
  add('&varrho;', #$03F1);
  add('&varsigma;', #$03C2);
  add('&varsubsetneq;', #$228A#$FE00);
  add('&varsubsetneqq;', #$2ACB#$FE00);
  add('&varsupsetneq;', #$228B#$FE00);
  add('&varsupsetneqq;', #$2ACC#$FE00);
  add('&vartheta;', #$03D1);
  add('&vartriangleleft;', #$22B2);
  add('&vartriangleright;', #$22B3);
  add('&vcy;', #$0432);
  add('&vdash;', #$22A2);
  add('&vee;', #$2228);
  add('&veebar;', #$22BB);
  add('&veeeq;', #$225A);
  add('&vellip;', #$22EE);
  add('&verbar;', #$007C);
  add('&vert;', #$007C);
  add('&vfr;', #$D835#$DD33);
  add('&vltri;', #$22B2);
  add('&vnsub;', #$2282#$20D2);
  add('&vnsup;', #$2283#$20D2);
  add('&vopf;', #$D835#$DD67);
  add('&vprop;', #$221D);
  add('&vrtri;', #$22B3);
  add('&vscr;', #$D835#$DCCB);
  add('&vsubnE;', #$2ACB#$FE00);
  add('&vsubne;', #$228A#$FE00);
  add('&vsupnE;', #$2ACC#$FE00);
  add('&vsupne;', #$228B#$FE00);
  add('&vzigzag;', #$299A);
  add('&wcirc;', #$0175);
  add('&wedbar;', #$2A5F);
  add('&wedge;', #$2227);
  add('&wedgeq;', #$2259);
  add('&weierp;', #$2118);
  add('&wfr;', #$D835#$DD34);
  add('&wopf;', #$D835#$DD68);
  add('&wp;', #$2118);
  add('&wr;', #$2240);
  add('&wreath;', #$2240);
  add('&wscr;', #$D835#$DCCC);
  add('&xcap;', #$22C2);
  add('&xcirc;', #$25EF);
  add('&xcup;', #$22C3);
  add('&xdtri;', #$25BD);
  add('&xfr;', #$D835#$DD35);
  add('&xhArr;', #$27FA);
  add('&xharr;', #$27F7);
  add('&xi;', #$03BE);
  add('&xlArr;', #$27F8);
  add('&xlarr;', #$27F5);
  add('&xmap;', #$27FC);
  add('&xnis;', #$22FB);
  add('&xodot;', #$2A00);
  add('&xopf;', #$D835#$DD69);
  add('&xoplus;', #$2A01);
  add('&xotime;', #$2A02);
  add('&xrArr;', #$27F9);
  add('&xrarr;', #$27F6);
  add('&xscr;', #$D835#$DCCD);
  add('&xsqcup;', #$2A06);
  add('&xuplus;', #$2A04);
  add('&xutri;', #$25B3);
  add('&xvee;', #$22C1);
  add('&xwedge;', #$22C0);
  add('&yacute;', #$00FD);
  add('&yacy;', #$044F);
  add('&ycirc;', #$0177);
  add('&ycy;', #$044B);
  add('&yen;', #$00A5);
  add('&yfr;', #$D835#$DD36);
  add('&yicy;', #$0457);
  add('&yopf;', #$D835#$DD6A);
  add('&yscr;', #$D835#$DCCE);
  add('&yucy;', #$044E);
  add('&yuml;', #$00FF);
  add('&zacute;', #$017A);
  add('&zcaron;', #$017E);
  add('&zcy;', #$0437);
  add('&zdot;', #$017C);
  add('&zeetrf;', #$2128);
  add('&zeta;', #$03B6);
  add('&zfr;', #$D835#$DD37);
  add('&zhcy;', #$0436);
  add('&zigrarr;', #$21DD);
  add('&zopf;', #$D835#$DD6B);
  add('&zscr;', #$D835#$DCCF);
  add('&zwj;', #$200D);
  add('&zwnj;', #$200C);
end;

end.
